
var Fresh; if (!Fresh) Fresh = {};

Fresh.PreferencesWindow = function (preferences) {
	this.preferences = preferences;
	this.runInBackground = new Ext.form.Checkbox({
		fieldLabel: 'Refresh in background',
		checked: preferences.getPreference('runInBackground')
	});
	this.refreshTimeout = new Ext.form.TextField({
		fieldLabel: 'Refresh timeout (minutes)',
		width: 50,
		value: preferences.getPreference('refreshTimeout')/60000,
		validationEvent: false,
		validateOnBlur: false,
		msgTarget: 'under',		
		listeners:{
			valid: this.syncShadow,
			invalid: this.syncShadow,
			scope: this
		}, 
	});
	this.startAtLogin = new Ext.form.Checkbox({
		fieldLabel: 'Start at login',
		checked: preferences.getPreference('startAtLogin')
	});
	this.startMinimized = new Ext.form.Checkbox({
		fieldLabel: 'Start minimized',
		checked: preferences.getPreference('startMinimized')
	});
	
	this.minimizeToTray = new Ext.form.Checkbox({
		fieldLabel: 'Minimize to tray',
		checked: preferences.getPreference('minimizeToTray')
	});
	this.showNotifications = new Ext.form.Checkbox({
		fieldLabel: 'Show notifications',
		checked: preferences.getPreference('showNotifications')
	});
	
	this.form = new Ext.FormPanel({
		labelWidth: 130,
		items: [this.runInBackground, this.refreshTimeout, this.startAtLogin, this.startMinimized, this.minimizeToTray, this.showNotifications],
		border: false,
		bodyStyle: 'background: transparent;padding: 10px;',
	});
	Fresh.PreferencesWindow.superclass.constructor.call(this, {
		title: 'Preferences',
		autoHeight: true,
		plain: true,
		modal: true,
		hidden: false,
		closeAction: 'hide',
		buttons: [{
			text: 'Save',
			handler: this.onSave,
			scope: this
		}, {
			text: 'Cancel',
			handler: this.hide.createDelegate(this, [])	
		}],
		items: this.form		
	});
}

Ext.extend(Fresh.PreferencesWindow, Ext.Window, {
	show: function() {
		this.runInBackground.setValue(this.preferences.getPreference('runInBackground'));
		this.showNotifications.setValue(this.preferences.getPreference('showNotifications'));
		this.startAtLogin.setValue(this.preferences.getPreference('startAtLogin'));
		this.startMinimized.setValue(this.preferences.getPreference('startMinimized'));
		this.minimizeToTray.setValue(this.preferences.getPreference('minimizeToTray'));
		this.refreshTimeout.setValue(this.preferences.getPreference('refreshTimeout')/60000);
		Fresh.PreferencesWindow.superclass.show.apply(this, arguments);
		Fresh.Reader.enableKeyMap(false);
	},
	
	hide: function() {
		Fresh.PreferencesWindow.superclass.hide.apply(this, arguments);
		Fresh.Reader.enableKeyMap(true);		
	},
	
	onSave: function() {
		var timeout = parseFloat(this.refreshTimeout.getValue());
		if (isNaN(timeout)) {
			this.markInvalid();
			return;
		}
		this.hide();
		return this.fireEvent('prefsave', {
				runInBackground: this.runInBackground.getValue(),
				refreshTimeout: timeout	* 60000,
				startAtLogin: this.startAtLogin.getValue(),
				startMinimized: this.startMinimized.getValue(),
				minimizeToTray: this.minimizeToTray.getValue(),
				showNotifications: this.showNotifications.getValue()	 
			});
	},
	
	markInvalid: function() {
		this.refreshTimeout.markInvalid('The specified timeout must be a number >= 0');
		this.el.unmask();	
	}
});